require 'ast'
require 'scanner'
require 'token'
require 'calcex'

class Parser
  def initialize(istream)
    @scan = Scanner.new(istream)
  end
  
  def parse()
    return Prog()
  end
  
  #private public protectec
  def Prog()
    result = Expr()
    t = @scan.getToken()
    
    if t.type != :eof then
      print "Expected EOF. Found ", t.type, ".\n"
      raise ParseError.new
    end
    
    return result
  end
  
  def Expr() 
    return RestExpr(Term())
  end
  
  def RestExpr(e) 
    t = @scan.getToken()
    
    if t.type == :add then
      return RestExpr(AddNode.new(e,Term()))
    end
    
    if t.type == :sub then
      return RestExpr(SubNode.new(e,Term()))
    end
    
    @scan.putBackToken()
    
    return e
  end
  
  def Term()
    RestTerm(Storable())
    # t = @scan.getToken()
    
    # if t.type == :number then
    #   val = t.lex.to_i
    #   return NumNode.new(val)
    # end
    
    # puts "Term not implemented\n"
    
    # raise ParseError.new
  end
  
  def RestTerm(e)
    t = @scan.getToken

    if t.type == :times
      return RestTerm(TimesNode.new(e, Storable()))
    end

    if t.type == :divide
      return RestTerm(DivideNode.new(e, Storable()))
    end

    @scan.putBackToken

    e
  end
  
  def Storable()
    factor = Factor()
    t = @scan.getToken

    if t.type == :keyword
      if t.lex == "S"
        return StoreNode.new(factor)
        raise ParserError.new()
      end
    end
    @scan.putBackToken()
    factor
  end
  
  def Factor() 
    t = @scan.getToken

    if t.type == :number
      return NumNode.new(t.lex.to_i)
    end

    if t.type == :keyword
      if t.lex == "R"
        return RecallNode.new
      end
      raise ParserError.new()
    end

    if t.type == :lparen
      expr = Expr()
      t = @scan.getToken
      if t.type == :rparen
        return expr
      end
      raise ParserError.new()
    end

    raise ParserError.new0
  end
end
